package com.example.sbcamel;

import org.apache.camel.Exchange;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.component.bean.BeanConstants;
import org.apache.camel.component.bean.validator.BeanValidationException;
import org.apache.camel.component.cxf.common.message.CxfConstants;
import org.apache.camel.component.zookeepermaster.policy.MasterRoutePolicy;
import org.apache.camel.model.dataformat.JsonLibrary;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import jakarta.annotation.PostConstruct;
import jakarta.ws.rs.core.Response;

@Component
public class CamelRouter extends RouteBuilder {
	private static final String createUserSql = "CREATE TABLE IF NOT EXISTS public.\"user\" (id BIGINT not null, "
			+ "name CHARACTER VARYING not null, PRIMARY KEY (id))";

	@Value("${app.queue-name}")
	private String queueName;

	@Value("${app.zookeeper.url}")
	private String zookeeperUrl;

	@Value("${app.zookeeper.group-name}")
	private String groupName;

	@Autowired
	private JdbcTemplate jdbcTemplate;

	@PostConstruct
	private void init() {
		org.apache.ibatis.logging.LogFactory.useSlf4jLogging();
		jdbcTemplate.execute(createUserSql);
	}

	@Override
	public void configure() throws Exception {
		// very raw way, just to handle the validation responses
		onException(BeanValidationException.class).handled(true)
				.setHeader(Exchange.HTTP_RESPONSE_CODE, constant(Response.Status.BAD_REQUEST.getStatusCode()))
				.setBody(simple("${exchangeProperty.CamelExceptionCaught.getMessage()}"));

		onException(jakarta.ws.rs.NotFoundException.class).handled(true)
				.setHeader(Exchange.HTTP_RESPONSE_CODE, constant(Response.Status.NOT_FOUND.getStatusCode()))
				.setBody(simple("${exchangeProperty.CamelExceptionCaught.getMessage()}"));

		from("cxfrs:/api?resourceClasses=" + UserService.class.getName() + "&bindingStyle=Default"
				+ "&providers=jaxrsProvider&loggingFeatureEnabled=true").to("log:cxfrs?showAll=true")
				.to("bean-validator:user").to("activemq6:queue:" + queueName).process(exchange -> {
					if (exchange.getMessage().getBody() != null && exchange.getMessage().getBody() instanceof byte[]
							&& new String((byte[]) exchange.getMessage().getBody()).equals("null")) {
						if (exchange.getIn().getHeader(Exchange.HTTP_METHOD).equals("GET")) {
							throw new jakarta.ws.rs.NotFoundException();
						}
					}
				});

		MasterRoutePolicy masterRoutePolicy = new MasterRoutePolicy();
		masterRoutePolicy.setZooKeeperUrl(zookeeperUrl);
		masterRoutePolicy.setGroupName(groupName);

		from("activemq6:queue:" + queueName).autoStartup("false").routePolicy(masterRoutePolicy)
				.process(exchange -> exchange.getIn().setHeader(BeanConstants.BEAN_METHOD_NAME,
						exchange.getIn().getHeader(CxfConstants.OPERATION_NAME.toLowerCase())))
				.to("log:activemq?showAll=true").transacted("propagationRequired").choice()
				.when(header(BeanConstants.BEAN_METHOD_NAME).isEqualTo("updateUser"))
				.to("mybatis:com.example.sbcamel.mapper.UserMapper.updateUser?statementType=Update")
				.when(header(BeanConstants.BEAN_METHOD_NAME).isEqualTo("findUsers"))
				.to("mybatis:com.example.sbcamel.mapper.UserMapper.findUsers?statementType=SelectList")
				.when(header(BeanConstants.BEAN_METHOD_NAME).isEqualTo("findUser"))
				.to("mybatis:com.example.sbcamel.mapper.UserMapper.findUser?statementType=SelectOne")
				.end().marshal().json(JsonLibrary.Jackson);
	}

}